      SUBROUTINE EXPOLA(X,Y,SIG,NDATA,R0,D0,COORD,
     &                  SRAD1,SRADN,FUN1,FUNN,IVAL,ICALL,VALUE,LUOUT)
********************************************************************************
*                                                                              *
*  Copyright (C) Dage Sundholm, sundholm@chem.helsinki.fi                      *
*  Department of Chemistry, University of Helsinki                             *
*  P.O. Box 55, FIN-00014 Helsinki, Finland                                    *
*                                                                              *
*  This program is free software; you can redistribute it and/or               *
*  modify it under the terms of the GNU General Public License                 *
*  as published by the Free Software Foundation; either version 2              *
*  of the License, or (at your option) any later version.                      *
*                                                                              *
*  This program is distributed in the hope that it will be useful,             *
*  but WITHOUT ANY WARRANTY; without even the implied warranty of              *
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
*  GNU General Public License for more details.                                *
*                                                                              *
*  You should have received a copy of the GNU General Public License           *
*  along with this program; if not, write to the Free Software                 *
*  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. *
*                                                                              *
********************************************************************************
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      SAVE R01,D01,ALPHA1,DE1,SHIFA1,SHIFB1
      SAVE R02,D02,ALPHA2,DE2,SHIFA2,SHIFB2
      SAVE C0,C1,C2,C3,C4,SHIFA3,SHIFB3 
      
      DIMENSION X(NDATA),Y(NDATA),SIG(NDATA)  

* Input  :   X      = Full coordinate vector
*        :   Y      = Full function vector
*        :   SIG    = Weight vector (=1.0 for all)
*        :   NDATA  = Number of sampling points
*        :   R0     = Initial guess for R0
*        :   D0     = Initial guess for D0
*        :   COORD  = Current coordinate
*        :   SRAD1  = Coordinate of the first sample point
*        :   SRADN  = Coordinate of the last sample point 
*        :   FUN1   = Function value of the first sample point
*        :   FUNN   = Function value of the last sample point
*        :   IVAL   = Select function number (1,2,or 3)
*        :   ICALL  = First call or not
*
* Output :   VALUE  = The extrapolated value of the given function
*
* Written by Dage Sundholm 28.6.1995
*
*
* The first call should create the extrapolation function 
* The expansion coefficients are save for future use
      IF(ICALL.EQ.1) THEN
* Create extrapolation function for potential function 1
        IF(IVAL.EQ.1) THEN 
          CALL MORASS(X,Y,SIG,NDATA,R0,D0,ALPHA,DE,LUOUT)
          R01=R0
          D01=D0
          ALPHA1=ALPHA
          DE1=DE
          SHIFA1=FUN1-DE*(1.D0-EXP(-ALPHA*(SRAD1-R0)))**2-D0
          SHIFB1=FUNN-DE*(1.D0-EXP(-ALPHA*(SRADN-R0)))**2-D0
        END IF
* Create extrapolation function for potential function 2
        IF(IVAL.EQ.2) THEN 
          CALL MORASS(X,Y,SIG,NDATA,R0,D0,ALPHA,DE,LUOUT)
          R02=R0
          D02=D0
          ALPHA2=ALPHA
          DE2=DE
          SHIFA2=FUN1-DE*(1.D0-EXP(-ALPHA*(SRAD1-R0)))**2-D0
          SHIFB2=FUNN-DE*(1.D0-EXP(-ALPHA*(SRADN-R0)))**2-D0
        END IF
* Create extrapolation function for property function
        IF(IVAL.EQ.3) THEN 
          CALL PROPSS(X,Y,SIG,NDATA,R01,ALPHA1,DE1,C0,C1,C2,C3,C4,LUOUT)
          CO1=SRAD1-R01
          WEIG=WEIGHT(CO1,ALPHA1,DE1)
          SHIFA3=FUN1-(C0+CO1*(C1+CO1*(C2+CO1*(C3+CO1*C4))))*WEIG
          SHIFA3=SHIFA3/WEIG
          fun1fu=(SHIFA3+C0+CO1*(C1+CO1*(C2+CO1*(C3+CO1*C4))))*WEIG
          CON=SRADN-R01
          WEIG=WEIGHT(CON,ALPHA1,DE1)
          SHIFB3=FUNN-(C0+CON*(C1+CON*(C2+CON*(C3+CON*C4))))*WEIG
          SHIFB3=SHIFB3/WEIG
        END IF
      END IF
      IF(IVAL.EQ.1) THEN  
        IF(COORD.LT.SRAD1) THEN 
          VALUE=DE1*(1.D0-EXP(-ALPHA1*(COORD-R01)))**2+D01+SHIFA1 
        ELSE
          VALUE=DE1*(1.D0-EXP(-ALPHA1*(COORD-R01)))**2+D01+SHIFB1
        END IF
        RETURN
      END IF 
      IF(IVAL.EQ.2) THEN
        IF(COORD.LT.SRAD1) THEN 
          VALUE=DE2*(1.D0-EXP(-ALPHA2*(COORD-R02)))**2+D02+SHIFA2
        ELSE
          VALUE=DE2*(1.D0-EXP(-ALPHA2*(COORD-R02)))**2+D02+SHIFB2
        END IF
        RETURN
      END IF 
      IF(IVAL.EQ.3) THEN
        IF(COORD.LT.SRAD1) THEN 
          COO=COORD-R01
          WEIG=WEIGHT(COO,ALPHA1,DE1)
          VALUE=(SHIFA3+C0+COO*(C1+COO*(C2+COO*(C3+COO*C4))))*WEIG
        ELSE
          COO=COORD-R01
          WEIG=WEIGHT(COO,ALPHA1,DE1)
          VALUE=(SHIFB3+C0+COO*(C1+COO*(C2+COO*(C3+COO*C4))))*WEIG
        END IF
        RETURN
      END IF 

      WRITE(6,*) ' WRONG OPTION IN EXPOLA'
      WRITE(6,*) ' IVAL = ',IVAL
      STOP

      END 
