      SUBROUTINE FINMIN(X,Y,DYDX,D2YDX2,NMESS,X0,Y0)
********************************************************************************
*                                                                              *
*  Copyright (C) Dage Sundholm, sundholm@chem.helsinki.fi                      *
*  Department of Chemistry, University of Helsinki                             *
*  P.O. Box 55, FIN-00014 Helsinki, Finland                                    *
*                                                                              *
*  This program is free software; you can redistribute it and/or               *
*  modify it under the terms of the GNU General Public License                 *
*  as published by the Free Software Foundation; either version 2              *
*  of the License, or (at your option) any later version.                      *
*                                                                              *
*  This program is distributed in the hope that it will be useful,             *
*  but WITHOUT ANY WARRANTY; without even the implied warranty of              *
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
*  GNU General Public License for more details.                                *
*                                                                              *
*  You should have received a copy of the GNU General Public License           *
*  along with this program; if not, write to the Free Software                 *
*  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. *
*                                                                              *
********************************************************************************
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION X(NMESS),Y(NMESS),DYDX(NMESS),D2YDX2(NMESS)
*
* Find the minimum of the spline approximation
* Written by Dage Sundholm 26.6.1995

* The minimum is located between KMESS and KMESS+1
* Find KMESS
      KMESS=0
      IF(DYDX(1).GT.0.D0) THEN
        DO 100 I=2,NMESS
          IF(DYDX(I).LT.0.D0.AND.KMESS.EQ.0) KMESS=I
100     CONTINUE
      END IF

      IF(DYDX(1).LT.0.D0) THEN
        DO 200 I=2,NMESS
          IF(DYDX(I).GT.0.D0.AND.KMESS.EQ.0) KMESS=I
200     CONTINUE
      END IF

* The derivative at the first point is too small
* Try the next sample point
      IF(ABS(DYDX(1)).LT.1.D-05) THEN
        KMESS=0
        IF(DYDX(2).GT.0.D0) THEN
          DO 110 I=3,NMESS
            IF(DYDX(I).LT.0.D0.AND.KMESS.EQ.0) KMESS=I
110       CONTINUE
        END IF
        IF(DYDX(2).LT.0.D0) THEN
          DO 210 I=3,NMESS
            IF(DYDX(I).GT.0.D0.AND.KMESS.EQ.0) KMESS=I
210       CONTINUE
        END IF
      END IF
* one step back to fulfil the condition of the first line
      KMESS=KMESS-1

* construct the second-order equation
      C1=(Y(KMESS+1)-Y(KMESS))/(X(KMESS+1)-X(KMESS))
      C2=-(X(KMESS+1)-X(KMESS))*D2YDX2(KMESS)/6.D0
      C3=(X(KMESS+1)-X(KMESS))*D2YDX2(KMESS+1)/6.D0
     
* Quadratic curve ?
      X000=21.213456789D0 
      IF(ABS(C2+C3).LT.1.D-07) THEN 
       IF(Y(KMESS+1).LT.Y(KMESS)) THEN
         X000=X(KMESS+1)
         Y000=Y(KMESS+1)
       ELSE
         X000=X(KMESS)
         Y000=Y(KMESS) 
       END IF
      END IF

      IF(X000.EQ.21.213456789D0) THEN 
       D2=3.D0*(C2+C3)
       D1=-6.D0*C3
       D0=2.D0*C3-C2+C1

* solve the D2*A**2 + D1*A + D0 = 0, for A
       SQ=D1*D1-4.D0*D2*D0

       IF(SQ.LT.0.D0) THEN
         WRITE(6,*) ' You have some serious problems in FINMIN'
         WRITE(6,*) ' I cannot go on '
         STOP
       END IF
 
       AP=-D1+SQRT(SQ)
       AP=AP/2.D0/D2
       AM=-D1-SQRT(SQ)
       AM=AM/2.D0/D2
       X0=1.234567890D30

* Select the correct root and test it
       IF(AP.GT.0.D0.AND.AP.LT.1.D0) THEN
         X0=AP*(X(KMESS)-X(KMESS+1))+X(KMESS+1)
         A=AP
       END IF
       IF(AM.GT.0.D0.AND.AM.LT.1.D0) THEN
        X0=AM*(X(KMESS)-X(KMESS+1))+X(KMESS+1)
         A=AM
       END IF

       IF(X0.EQ.1.234567890D30) THEN
         WRITE(6,*) ' X0 is equal to its initial value'
         WRITE(6,*) ' That means that FINMIN failed to find something'
         WRITE(6,*) ' reasonable, I must stop here '
         STOP
       END IF
 
* Find the coefficients of the spline expansion in this interval
       B=1.D0-A
       C=A*(A*A-1.D0)/6.D0*(X(KMESS+1)-X(KMESS))**2
       D=B*(B*B-1.D0)/6.D0*(X(KMESS+1)-X(KMESS))**2
       Y0=A*Y(KMESS)+B*Y(KMESS+1)+C*D2YDX2(KMESS)+D*D2YDX2(KMESS+1)
      ELSE
       X0=X000
       Y0=Y000
      END IF

      RETURN
      END
