      SUBROUTINE FIRDER(X,Y,D2YDX2,NMESS,DYDX)
********************************************************************************
*                                                                              *
*  Copyright (C) Dage Sundholm, sundholm@chem.helsinki.fi                      *
*  Department of Chemistry, University of Helsinki                             *
*  P.O. Box 55, FIN-00014 Helsinki, Finland                                    *
*                                                                              *
*  This program is free software; you can redistribute it and/or               *
*  modify it under the terms of the GNU General Public License                 *
*  as published by the Free Software Foundation; either version 2              *
*  of the License, or (at your option) any later version.                      *
*                                                                              *
*  This program is distributed in the hope that it will be useful,             *
*  but WITHOUT ANY WARRANTY; without even the implied warranty of              *
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
*  GNU General Public License for more details.                                *
*                                                                              *
*  You should have received a copy of the GNU General Public License           *
*  along with this program; if not, write to the Free Software                 *
*  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. *
*                                                                              *
********************************************************************************
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
*
* Calculate the first derivative in each sample point
* using using the cubic-spline representation of the function and its
* second derivative
* Numerical Recipes page 87
* Coded by Dage Sundholm, 23.6.1995
*
*
*  INPUT:  X(I)      : X-value for the sample points
*          Y(I)      : Y-value for the sample points
*          NMESS     : Number of sample points
*          D2YDX2(I) : Second derivative in each sample point
*
*  OUTPUT: DYDX(I)   : First derivative in each sample point

      DIMENSION X(NMESS),Y(NMESS),D2YDX2(NMESS),DYDX(NMESS)
*
      A=1.D0
      B=0.D0
      A3P2=-(3.D0*A*A-1.D0)/6.D0
      B3P2=(3.D0*B*B-1.D0)/6.D0
      DO 100 I=1,NMESS-1
        C1=(Y(I+1)-Y(I))/(X(I+1)-X(I))
        C2=(X(I+1)-X(I))*D2YDX2(I)*A3P2
        C3=(X(I+1)-X(I))*D2YDX2(I+1)*B3P2
        DYDX(I)=C1+C2+C3
100   CONTINUE

* last point
      A=0.D0
      B=1.D0
      A3P2=-(3.D0*A*A-1.D0)/6.D0
      B3P2=(3.D0*B*B-1.D0)/6.D0
      C1=(Y(NMESS)-Y(NMESS-1))/(X(NMESS)-X(NMESS-1))
      C2=(X(NMESS)-X(NMESS-1))*D2YDX2(NMESS-1)*A3P2
      C3=(X(NMESS)-X(NMESS-1))*D2YDX2(NMESS)*B3P2
      DYDX(NMESS)=C1+C2+C3

      RETURN
      END
