      SUBROUTINE HASMAT(NPOINT,NCELL,NDIM,IVAR,RS,H,SQPOT,
     &                  AID1,AID2,AID3,AID4,AID5,AID6,
     &                  AIA1,AIA2,AIA3,AIA4,AMAT,VMAT,CMAT,TMAT,
     &                  EMAT,WRK,QUANTU,REDMAS,PROPER,
     &                  MINVVA,MAXVVA,WMAT,FMAT,EIGE,SMAT,PSHIFT,
     &                  ELEVEL,PROPTY,MINJVA,MAXJVA)
********************************************************************************
*                                                                              *
*  Copyright (C) Dage Sundholm, sundholm@chem.helsinki.fi                      *
*  Department of Chemistry, University of Helsinki                             *
*  P.O. Box 55, FIN-00014 Helsinki, Finland                                    *
*                                                                              *
*  This program is free software; you can redistribute it and/or               *
*  modify it under the terms of the GNU General Public License                 *
*  as published by the Free Software Foundation; either version 2              *
*  of the License, or (at your option) any later version.                      *
*                                                                              *
*  This program is distributed in the hope that it will be useful,             *
*  but WITHOUT ANY WARRANTY; without even the implied warranty of              *
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
*  GNU General Public License for more details.                                *
*                                                                              *
*  You should have received a copy of the GNU General Public License           *
*  along with this program; if not, write to the Free Software                 *
*  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. *
*                                                                              *
********************************************************************************
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
 
*   Construct the H and S matrices and diagonalize the generalized
*   eigenvalue problem Hx = ESx
 
*   INPUT:
*        NPOINT    = NUMBER OF LAGRANGE INTERPOLATION POLYNOMIALS PER
*                    CELL
*        NCELL     = NUMBER OF CELLS
*        NDIM      = TOTAL NUMBER OF POINTS
*        IVAR      = EXPANSION POINT FOR LIPS IN EACH CELL
*        RS        = START COORDINATE FOR THE CELLS
*        H         = STEP LENGTHS OF THE CELLS
*        SQPOT     = SQUARE ROOT OF THE POTENTIAL
*        AID1      = HELP ARRAY #1
*        AID2      = HELP ARRAY #2
*        AID3      = HELP ARRAY #3
*        AID4      = HELP ARRAY #4
*        AID5      = HELP ARRAY #5
*        AID6      = HELP ARRAY #6
*        AIA1      = PACKED INTEGRAL ARRAY #1
*        AIA2      = PACKED INTEGRAL ARRAY #2
*        AIA3      = PACKED INTEGRAL ARRAY #3
*        AIA4      = PACKED INTEGRAL ARRAY #4
*        AMAT      = THE FULL LAPLACIAN MATRIX
*        VMAT      = A FULL POTENTIAL MATRIX
*        CMAT      = A FULL HELP MATRIX
*        TMAT      = A FULL HELP MATRIX
*        EMAT      = A FULL HELP MATRIX
*        WRK       = A HELP ARRAY
*        QUANTU    = ROTATION QUANTUM NUMBER (J) 
*        REDMAS    = REDUCED MASS
*        PROPER    = THE PROPERTY VECTOR 
*        MINVVA    = FIRST VIBRATIONAL LEVEL (1 = FIRST)
*        MAXVVA    = LAST VIBRATIONAL LEVEL
*        WMAT      = A FULL HELP MATRIX
*        FMAT      = A FULL HELP MATRIX
*        EIGE      = ROVIBRATIONAL EIGENVECTORS 
*        SMAT      = OVERLAP MATRIX 
*        PSHIFT    = AN ENERGY SHIFT
*        ELEVEL    = THE ENERGY LEVEL
*        PROPTY    = THE AVERAGED PROPERTY
*        MINJVA    = FIRST ROTATIONAL LEVEL (0 = FIRST)
*        MAXJVA    = LAST ROTATIONAL LEVEL

 
      COMMON/IO/LUIN,LUOUT
      DIMENSION RS(NCELL),H(NCELL),SQPOT(NCELL*(NPOINT-1)-1)
      DIMENSION AID1(100),AID2(100),AID3(100),AID4(100)
      DIMENSION AID5(NPOINT*NPOINT),AID6(NPOINT*NPOINT*(NPOINT-1))
      DIMENSION AIA1(NPOINT*NPOINT*NCELL),AIA2(NPOINT*NPOINT*NCELL)
      DIMENSION AIA3(NPOINT*NPOINT*NCELL),AIA4(NPOINT*NPOINT*NCELL)
      DIMENSION AMAT((NDIM-2)*(NDIM-2)),VMAT((NDIM-2)*(NDIM-2))
      DIMENSION CMAT((NDIM-2)*(NDIM-2)),TMAT((NDIM-2)*(NDIM-2))
      DIMENSION EMAT((NDIM-2)*(NDIM-2)),WRK(NDIM-2),PROPER(NDIM-2)
      DIMENSION WMAT((NDIM-2)*(NDIM-2)),FMAT((NDIM-2)*(NDIM-2))
      DIMENSION EIGE((NDIM-2)*(NDIM-2)),SMAT((NDIM-2)*(NDIM-2))
      DIMENSION ELEVEL(MINVVA:MAXVVA,MINJVA:MAXJVA)
      DIMENSION PROPTY(MINVVA:MAXVVA,MINJVA:MAXJVA)

* Debugging switch (=0) little, (=2) some more, (=5) some more 
* (=11) = a Hydrogen-like problem is solved
      
      IDEBUG = 0

      IF(IDEBUG.GT.0) THEN
        WRITE(LUOUT,*) ' IDEBUG not equal ZERO in HASMAT'
      END IF

* Compute ingredients to the matrices in packed form
      CALL MATPOI(NPOINT,NCELL,NDIM,IVAR,AID1,AID2,AID3,AID4,
     &            AID5,AID6,AIA1,AIA2,AIA3,AIA4,RS,H)
 
* Setup the A, S, V and W matrices
      CALL SETMAT(AMAT,SMAT,VMAT,WMAT,AIA1,AIA2,AIA3,AIA4,
     &            NCELL,NPOINT,NDIM)               

      IF(IDEBUG.GT.10) THEN
*  Check the matrices
        WRITE(LUOUT,'(A)') ' The debugging switch is on in HASMAT'
        WRITE(LUOUT,'(A)') ' The program will enter TESTMA'
        CALL TESTMA(AMAT,SMAT,WMAT,VMAT,NCELL,NPOINT,NDIM,
     &              CMAT,TMAT,RS,H)
      END IF
           
* Construct the total Hamilton matrix by adding the kinetic and 
* potential terms  
      CALL TOTHAM(AMAT,SMAT,WMAT,VMAT,CMAT,TMAT,SQPOT,RS,H,
     &            QUANTU,REDMAS,NCELL,NPOINT,NDIM)
 
* Transform the general case to orthonormal eigenvalue problem Hx=Ex
* A safety copy of SMAT, it will be destroyed in TRANSH
* All original A,V,W and S matrices are intact
* I put the transformation matrix into CMAT
      CALL VMOV(SMAT,1,FMAT,1,(NDIM-2)*(NDIM-2))
      CALL TRANSH(NDIM-2,TMAT,FMAT,CMAT,WRK)

* The eigenvalue solver 
      LDUMMY=0
      CALL EIG(TMAT,EIGE,LDUMMY,NDIM-2,0)

* The energy shift
      SHIFT=1.D0
      DO 100 I=1,NDIM-2
       TMAT(I+(I-1)*(NDIM-2))=TMAT(I+(I-1)*(NDIM-2))-SHIFT
100   CONTINUE

* Backtransformation of the eigenvectors to the Lagrange interpolation
* function basis
* The final eigenvectors are in EMAT and the corresponding
* eigenvalues are in the diagonal of TMAT  
      CALL MATML4(EMAT,CMAT,EIGE,
     &            NDIM-2,NDIM-2,NDIM-2,NDIM-2,NDIM-2,NDIM-2,0)

      IF(IDEBUG.GE.5) THEN 
*       WRITE(LUOUT,*)
*       WRITE(LUOUT,'(A)') ' The energies (au) '
*       WRITE(LUOUT,'(A,1X,I3)') ' Calculated for J= ',NINT(QUANTU)
*       WRITE(LUOUT,*)
*       WRITE(LUOUT,1000) 
*    &  (TMAT(I+(I-1)*(NDIM-2)),I=MINVVA,MAXVVA)
*       WRITE(LUOUT,*)
        IF(NINT(QUANTU).EQ.MINJVA) THEN
          WRITE(LUOUT,'(A)') ' The energies (au) '
          WRITE(LUOUT,*)
        END IF
        MAXWWA=MIN(5,MAXVVA)
        WRITE(LUOUT,*)
        WRITE(LUOUT,'(A,3X,I4,3X,5(D13.6,2X))') 'ENE',NINT(QUANTU),
     &  (TMAT(I+(I-1)*(NDIM-2)),I=MINVVA,MAXWWA)
        IF(MAXVVA.GT.5) THEN 
          WRITE(LUOUT,'(A,10X,5(D13.6,2X))') 'ENE',
     &   (TMAT(I+(I-1)*(NDIM-2)),I=MAXWWA,MAXVVA)
        END IF 
      ELSE 
*       WRITE(LUOUT,*)
*       WRITE(LUOUT,'(A)') ' The vibrational energies (cm-1) '
*       WRITE(LUOUT,'(A,1X,I3)') ' Calculated for J= ',NINT(QUANTU)
*       WRITE(LUOUT,*)
* 1 au = 219474.625 cm-1 is used
*       WRITE(LUOUT,1000)
*    &  (TMAT(I+(I-1)*(NDIM-2))*219474.625,I=MINVVA,MAXVVA)
        IF(NINT(QUANTU).EQ.MINJVA) THEN
          WRITE(LUOUT,*)
          WRITE(LUOUT,*)
          WRITE(LUOUT,'(A)') ' The vibrational energies (cm-1) '
          WRITE(LUOUT,*)
        END IF        
        MAXWWA=MIN(5,MAXVVA)
        WRITE(LUOUT,*)
        WRITE(LUOUT,'(A,3X,I4,3X,5(D13.6,2X))') 'ENE',NINT(QUANTU),
     &  (TMAT(I+(I-1)*(NDIM-2))*219474.625,I=MINVVA,MAXWWA)
        IF(MAXVVA.GT.5) THEN
          WRITE(LUOUT,'(A,10X,5(D13.6,2X))') 'ENE',
     &   (TMAT(I+(I-1)*(NDIM-2))*219474.625,I=MAXWWA,MAXVVA)
        END IF
      END IF 
*1000  FORMAT(5(D13.6,2X))

*  Save the energy levels in elevel
        DO 200 IVIB=MINVVA,MAXVVA
          ELEVEL(IVIB,NINT(QUANTU))=
     &    TMAT(IVIB+(IVIB-1)*(NDIM-2))*219474.625D0
200     CONTINUE

*  Print the MAXVVA first eigenvectors 
      IF(IDEBUG.GT.1) THEN 
        ILAST=MAXVVA
        CALL TRMASA(EMAT,NDIM-2)
        CALL PRIWF(EMAT,ILAST,NDIM)
        CALL TRMASA(EMAT,NDIM-2) 
      END IF

* Integration for the average property
      CALL VCLR(TMAT,1,NDIM*NDIM)
      CALL VCLR(AMAT,1,NDIM*NDIM)
      CALL AVERAG(EMAT,PROPER,NDIM,SMAT,AMAT,TMAT,QUANTU,
     &            MINVVA,MAXVVA,MINJVA,MAXJVA,PROPTY)
 
      RETURN
      END
