      SUBROUTINE INTGPD(PT,NPOINT,PP,PPP,R,A1,H,NCELL,NDIM)
********************************************************************************
*                                                                              *
*  Copyright (C) Dage Sundholm, sundholm@chem.helsinki.fi                      *
*  Department of Chemistry, University of Helsinki                             *
*  P.O. Box 55, FIN-00014 Helsinki, Finland                                    *
*                                                                              *
*  This program is free software; you can redistribute it and/or               *
*  modify it under the terms of the GNU General Public License                 *
*  as published by the Free Software Foundation; either version 2              *
*  of the License, or (at your option) any later version.                      *
*                                                                              *
*  This program is distributed in the hope that it will be useful,             *
*  but WITHOUT ANY WARRANTY; without even the implied warranty of              *
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
*  GNU General Public License for more details.                                *
*                                                                              *
*  You should have received a copy of the GNU General Public License           *
*  along with this program; if not, write to the Free Software                 *
*  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. *
*                                                                              *
********************************************************************************
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
 
C CALCULATE SINGLE LIP DERIVATIVE INTEGRALS
 
C INPUT:
C        PT        = LIPS IN EACH INTERVAL
C        NPOINT    = NUMBER OF LIPS PER CELL
C        PP        = WORKING SPACE
C        PPP       = WORKING SPACE
C        R         = WORKING SPACE
C        NCELL     = NUMBER OF CELLS
C        H         = STEP LENGTH IN EACH CELL
C        NDIM      = TOTAL NUMBER OF POINTS
C OUTPUT:
C        A1        = INTEGRALS IN A (NPOINT,NPOINT,NCELL) MATRIX
 
      DIMENSION PT(NPOINT,NPOINT-1,NPOINT)
      DIMENSION A1(NPOINT,NPOINT,NCELL),H(NCELL)
      DIMENSION PP(100),PPP(100),R(100)
 
C RESET
 
      CALL VCLR(R,1,100)
      CALL VCLR(PP,1,100)
      CALL VCLR(PPP,1,100)
      CALL VCLR(A1,1,NPOINT*NPOINT*NCELL)
 
C COMPUTE INTEGRALS FOR ONE CELL. The others are scaled from these.
 
      DO 100 ILIP=1,NPOINT
        DO 100 JLIP=ILIP,NPOINT
 
          DO 100 INT=1,NPOINT-1
            CALL DERPOL(PT(1,INT,ILIP),PP,NPOINT)
            CALL VMOV(PT(1,INT,JLIP),1,PPP,1,NPOINT)
 
            CALL TPOLMU(PP,NPOINT-1,PPP,NPOINT,R,2*NPOINT-2)
            CALL INTEG(R,2*NPOINT-2,S,0.5D0)
 
            A1(ILIP,JLIP,1)=A1(ILIP,JLIP,1)+S
            A1(JLIP,ILIP,1)=A1(ILIP,JLIP,1)
100   CONTINUE
 
C scale and scatter out non-vanishing elements (in a packed form)
 
      DO 200 ICELL=NCELL,1,-1
        WEIGHT=1.D0
        DO 200 IP=1,NPOINT
          DO 200 JP=1,NPOINT
            A1(IP,JP,ICELL)=A1(IP,JP,1)*WEIGHT
200   CONTINUE
 
C reset matrix element for the first and last points (boundary condition
 
      DO 300 I=1,NPOINT
        A1(1,I,1)=0.D0
        A1(I,1,1)=0.D0
300   CONTINUE 
 
      DO 400 I=1,NPOINT
        A1(NPOINT,I,NCELL)=0.D0
        A1(I,NPOINT,NCELL)=0.D0
400   CONTINUE
 
      RETURN
      END
