      SUBROUTINE INTGRP(PT,NPOINT,PP,PPP,R,A1,NCELL,RS,H,NDIM)
********************************************************************************
*                                                                              *
*  Copyright (C) Dage Sundholm, sundholm@chem.helsinki.fi                      *
*  Department of Chemistry, University of Helsinki                             *
*  P.O. Box 55, FIN-00014 Helsinki, Finland                                    *
*                                                                              *
*  This program is free software; you can redistribute it and/or               *
*  modify it under the terms of the GNU General Public License                 *
*  as published by the Free Software Foundation; either version 2              *
*  of the License, or (at your option) any later version.                      *
*                                                                              *
*  This program is distributed in the hope that it will be useful,             *
*  but WITHOUT ANY WARRANTY; without even the implied warranty of              *
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
*  GNU General Public License for more details.                                *
*                                                                              *
*  You should have received a copy of the GNU General Public License           *
*  along with this program; if not, write to the Free Software                 *
*  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. *
*                                                                              *
********************************************************************************
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
 
C CALCULATE DOUBLE LIP INTEGRALS TIMES 1/R
 
C INPUT:
C        PT        = LIPS IN EACH INTERVAL
C        NPOINT    = NUMBER OF LIPS PER CELL
C        PP        = WORKING SPACE
C        PPP       = WORKING SPACE
C        R         = WORKING SPACE
C        NCELL     = NUMBER OF CELLS
C        RS        = COORDINATE FOR START OF EACH CELL
C        H         = STEP LENGTH IN EACH CELL
C        NDIM      = TOTAL NUMBER OF POINTS
C OUTPUT:
C        A1        = INTEGRALS IN A (NPOINT,NPOINT,NCELL) MATRIX
 
      COMMON/IO/LUIN,LUOUT

      DIMENSION PT(NPOINT,NPOINT-1,NPOINT)
      DIMENSION PP(100),PPP(100),R(100),RS(NCELL),H(NCELL)
      DIMENSION A1(NPOINT,NPOINT,NCELL)
 
C MAX LENGTH OF EXPANSION AND CONVERGENCE CRITERION FOR EXPANSION
 
      MAXVRV=100
      CONLIM=1.D-10
 
C RESET
 
      CALL VCLR(R,1,100)
      CALL VCLR(PP,1,100)
      CALL VCLR(PPP,1,100)
      CALL VCLR(A1,1,NPOINT*NPOINT*NCELL)
 
C LOOP OVER CELLS
 
      DO 100 ICELL=1,NCELL
 
C IF ICELL = 1 SOME MATRIX ELEMENTS IS EQUAL TO ZERO
 
        LIPSTA=1
        IF(ICELL.EQ.1) THEN
          LIPSTA=2
        END IF
 
C LOOP OVER LIPS (DOUBLE LOOP)
 
        DO 100 ILIP=LIPSTA,NPOINT
          DO 100 JLIP=ILIP,NPOINT
 
C LOOP OVER INTERVALS IN CELL ICELL
 
              DO 100 INT=1,NPOINT-1
 
C ILIP TIMES JLIP INTO PP
 
                CALL TPOLMU(PT(1,INT,ILIP),NPOINT,PT(1,INT,JLIP),NPOINT,
     &                      PP,2*NPOINT-1)
 
C EXPANSION FOR 1/R (INCLUDING TEST OF LENGTH OF EXPANSION)
 
C FIRST CELL
 
                IF(ICELL.EQ.1) THEN
                  CALL VCLR(R,1,100)
                  R(1)=(2*INT-1)*H(1)*0.5D0
                  R(2)=H(1)
 
                  CALL POLDIV(PP,R,PPP,1,2*NPOINT-1)
                  NPAR=2*NPOINT-2
 
                ELSE
 
C BULK CELLS
 
                  CONR=1./(RS(ICELL)+(2*INT-1)*H(ICELL)*0.5D0)
                  ACONR=CONR*H(ICELL)
                  R(1)=1.D0
                  IVARV=1
 777                CONTINUE
                    IVARV=IVARV+1
                    IF(IVARV.GE.(MAXVRV-2*NPOINT+2)) THEN
                    WRITE(LUOUT,'(A)') 
     &              'PROBLEMS WITH DIVISION IN INTGRP'
                    WRITE(LUOUT,'(A)') 
     &              'PLEASE REDIMENSION THE ARRAYS'
                    STOP
                    END IF

                    R(IVARV)=-R(IVARV-1)*ACONR
                    IF(IVARV.EQ.(MAXVRV-2*NPOINT+2)) GO TO 779
                    IF(ABS(R(IVARV))*CONR.GT.CONLIM) GO TO 777
 
 779              CONTINUE
                  DO 778 ITERM=1,IVARV
 778              R(ITERM)=R(ITERM)*CONR
 
                  NPAR=2*NPOINT-2+IVARV
                  CALL TPOLMU(PP,2*NPOINT-1,R,IVARV,PPP,NPAR)
 
                END IF
 
C INTEGRATE PP FROM -0.5 TO 0.5
 
                CALL INTEG(PPP,NPAR,S,0.5D0)
                S=S*H(ICELL)
 
C ADDING TO GET TOTAL INTEGRAL
 
                A1(ILIP,JLIP,ICELL)=A1(ILIP,JLIP,ICELL)+S
                A1(JLIP,ILIP,ICELL)=A1(ILIP,JLIP,ICELL)
100   CONTINUE
 
      DO 300 I=1,NPOINT
        A1(1,I,1)=0.D0
        A1(I,1,1)=0.D0
300   CONTINUE
 
      DO 400 I=1,NPOINT
        A1(I,NPOINT,NCELL)=0.D0
        A1(NPOINT,I,NCELL)=0.D0
400   CONTINUE

*      DO 1100 NC=1,NCELL
*      DO 1100 I=1,npoint
*       DO 1100 J=1,npoint
*        WRITE(6,*) A1(I,J,NC)
*1100  continue 
        
 
      RETURN
      END
