      SUBROUTINE MORASS(X,Y,SIG,NDATA,R0,D0,ALF,DE,LUOUT)
********************************************************************************
*                                                                              *
*  Copyright (C) Dage Sundholm, sundholm@chem.helsinki.fi                      *
*  Department of Chemistry, University of Helsinki                             *
*  P.O. Box 55, FIN-00014 Helsinki, Finland                                    *
*                                                                              *
*  This program is free software; you can redistribute it and/or               *
*  modify it under the terms of the GNU General Public License                 *
*  as published by the Free Software Foundation; either version 2              *
*  of the License, or (at your option) any later version.                      *
*                                                                              *
*  This program is distributed in the hope that it will be useful,             *
*  but WITHOUT ANY WARRANTY; without even the implied warranty of              *
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
*  GNU General Public License for more details.                                *
*                                                                              *
*  You should have received a copy of the GNU General Public License           *
*  along with this program; if not, write to the Free Software                 *
*  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. *
*                                                                              *
********************************************************************************
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
*
* This subroutine fits a morse function
* to the potential energy curve data
* an initial guess for R0 and D0 are also provided.
* but all 4 parameters are optimized
* 
* Input    :   X     = Bond lengths
*          :   Y     = Potential curve 
*          :   SIG   = Weights (=1.0)
*          :   NDATA = Sample points
*          :   R0    = Initial guess for equlibrium bond length
*          :   D0    = Initial guess for equlibrium energy 
*          :   LUOUT = Print channel 
*
* Output   :   R0    = Equlibrium bond length for the Morse curve
*          :   D0    = Equlibrium energy for the Morse curve
*          :   ALF   = Exponential parameter for the Morse curve
*          :   DE    = Dissociation energy for the Morse curve
*
* Written by Dage Sundholm 27.6.1995
*
      PARAMETER (MFIT=4)
      PARAMETER (MA=4)
      PARAMETER (NCA=4)

      DIMENSION X(NDATA),Y(NDATA),SIG(NDATA)
      DIMENSION A(MA),LISTA(MA),AOLD(MA)
      DIMENSION COVAR(NCA,NCA),ALPHA(NCA,NCA)

* All sample points are equal
      DEGUES=1.D31
      DO 1 I=1,NDATA
        SIG(I)=1.D0
        DEGUES=MIN(Y(I),DEGUES)
1     CONTINUE
      DEGUES=Y(NDATA)-DEGUES
 
*  Initialization
      A(1)=DEGUES
      A(2)=1.D0
      A(3)=R0                 
      A(4)=D0                  
      AOLD(1)=A(1)
      AOLD(2)=A(2)
      AOLD(3)=A(3)
      AOLD(4)=A(4)

* optimization of them all
      DO 10 I=1,MA
        LISTA(I)=I
10    CONTINUE

* Initialization of the optimizing subroutine
      ALAMDA=-100.D0
      CALL MRQMIN(X,Y,SIG,NDATA,A,MA,LISTA,MFIT,
     &            COVAR,ALPHA,NCA,CHISQ,ALAMDA,1)

* Let's go
      ITER=0
999   CONTINUE
      CALL MRQMIN(X,Y,SIG,NDATA,A,MA,LISTA,MFIT,
     &            COVAR,ALPHA,NCA,CHISQ,ALAMDA,1)
      DIFF=0.D0
      DO 111 I=1,MA
        DIFF=DIFF+(A(I)-AOLD(I))**2
111   CONTINUE
      DIFF=SQRT(DIFF)
      AOLD(1)=A(1)
      AOLD(2)=A(2)
      AOLD(3)=A(3)
      AOLD(4)=A(4)
      IF(DIFF.LT.1.D-10) THEN
        ITER=ITER+1
      ELSE 
        ITER=0
      END IF
      IF(DIFF.GT.1.D-10.OR.ITER.LT.3) GO TO 999

* Some statistics about the fit (if you want)
* Actually I do not care too much now
      ALAMDA=0.D0
      CALL MRQMIN(X,Y,SIG,NDATA,A,MA,LISTA,MFIT,
     &            COVAR,ALPHA,NCA,CHISQ,ALAMDA,1)

      DIFFYS=0.D0
      DO 200 I=1,NDATA
        CALL MORSE(X(I),A,YY,AOLD,MA)
        DIFFY=Y(I)-YY
        DIFFYS=DIFFYS+DIFFY*DIFFY
200   CONTINUE
      DIFFYS=SQRT(DIFFYS)

      WRITE(LUOUT,*)
      WRITE(LUOUT,*) ' The parameters of the optimized Morse function'
      WRITE(LUOUT,*) 
      WRITE(LUOUT,*) '     De          Alpha         Re   ',
     &               '           E0             Fit                    '
      WRITE(LUOUT,*) 
      WRITE(LUOUT,'(2X,3(D11.5,2X),D17.11,1X,D11.5)') 
     &              A(1),A(2),A(3),A(4),DIFFYS 
      WRITE(LUOUT,*) 

* Copy to the parameters of the parameter list
      DE=A(1)
      ALF=A(2)
      R0=A(3)
      D0=A(4)

      RETURN
      END
