      SUBROUTINE PROPSS(X,Y,SIG,NDATA,R0,ALF,DE,C0,C1,C2,C3,C4,LUOUT) 
********************************************************************************
*                                                                              *
*  Copyright (C) Dage Sundholm, sundholm@chem.helsinki.fi                      *
*  Department of Chemistry, University of Helsinki                             *
*  P.O. Box 55, FIN-00014 Helsinki, Finland                                    *
*                                                                              *
*  This program is free software; you can redistribute it and/or               *
*  modify it under the terms of the GNU General Public License                 *
*  as published by the Free Software Foundation; either version 2              *
*  of the License, or (at your option) any later version.                      *
*                                                                              *
*  This program is distributed in the hope that it will be useful,             *
*  but WITHOUT ANY WARRANTY; without even the implied warranty of              *
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
*  GNU General Public License for more details.                                *
*                                                                              *
*  You should have received a copy of the GNU General Public License           *
*  along with this program; if not, write to the Free Software                 *
*  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. *
*                                                                              *
********************************************************************************
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      COMMON/PRPASS/ALFPS,DEPS
*
* This subroutine fits a polynomial of 4th order times a cut-off
* function (EXP(-X*X), X=R-R0) to the property function
* X=0 at R=R0 all 5 parameters are optimized. 
*
* Input  : X      =  Bond length 
*          Y      =  Property function 
*          SIG    =  Weights (= 1.0)
*          NDATA  =  Number of sample points
*          R0     =  Equilibrium bond length
*          ALF    =  Morse exponent factor
*          DE     =  Morse dissociation energy
*          LUOUT  =  Print channel

* Output : C0     =  The constant term
*        : C1     =  The linear term
*        : C2     =  The quadratic term
*        : C3     =  The qubic term
*        : C4     =  The fourth-order term
*
* Written by Dage Sundholm 28.6.1995
*
      PARAMETER (MFIT=5)
      PARAMETER (MA=5)
      PARAMETER (NCA=5)

      DIMENSION X(NDATA),Y(NDATA),SIG(NDATA)
      DIMENSION A(MA),LISTA(MA),AOLD(MA)
      DIMENSION COVAR(NCA,NCA),ALPHA(NCA,NCA)

* Copy Morse parameters for the property fit 
      R0PS=R0
      ALFPS=ALF
      DEPS=DE

* All sample points are equal
* Change from X=R to X=R-R0 for the fit 
      DO 1 I=1,NDATA
        X(I)=X(I)-R0
        SIG(I)=1.D0
1     CONTINUE

*  Initialization
      A(1)=1.D0
      A(2)=0.D0
      A(3)=0.D0
      A(4)=0.D0
      A(5)=0.D0
      AOLD(1)=A(1)
      AOLD(2)=A(2)
      AOLD(3)=A(3)
      AOLD(4)=A(4)
      AOLD(5)=A(5)

* optimization of them all
      DO 10 I=1,MA
        LISTA(I)=I
10    CONTINUE

* Initialization of the optimizing subroutine
      ALAMDA=-100.D0
      CALL MRQMIN(X,Y,SIG,NDATA,A,MA,LISTA,MFIT,
     &            COVAR,ALPHA,NCA,CHISQ,ALAMDA,2)

* Let's go
999   CONTINUE
      CALL MRQMIN(X,Y,SIG,NDATA,A,MA,LISTA,MFIT,
     &            COVAR,ALPHA,NCA,CHISQ,ALAMDA,2)
      DIFF=0.D0
      DO 111 I=1,MA
        DIFF=DIFF+(A(I)-AOLD(I))**2
111   CONTINUE
      DIFF=SQRT(DIFF)
      AOLD(1)=A(1)
      AOLD(2)=A(2)
      AOLD(3)=A(3)
      AOLD(4)=A(4)
      AOLD(5)=A(5)
      IF(DIFF.GT.1.D-10) GO TO 999

* Some statistics about the fit (if you want)
* Actually I do not care too much now
      ALAMDA=0.D0
      CALL MRQMIN(X,Y,SIG,NDATA,A,MA,LISTA,MFIT,
     &            COVAR,ALPHA,NCA,CHISQ,ALAMDA,2)

      DIFFYS=0.D0
      DO 200 I=1,NDATA
        CALL PROPFU(X(I),A,YY,AOLD,MA)
        DIFFY=Y(I)-YY
        DIFFYS=DIFFYS+DIFFY*DIFFY
200   CONTINUE
      DIFFYS=SQRT(DIFFYS)

      WRITE(LUOUT,*)
      WRITE(LUOUT,*) ' The parameters of the optimized property',
     &' function'
      WRITE(LUOUT,*) 
      WRITE(LUOUT,*) '      C0          C1           C2    ',
     &               '       C3          C4          Fit     '
      WRITE(LUOUT,*) 
      WRITE(LUOUT,'(2X,6(D11.5,2X))') A(1),A(2),A(3),A(4),A(5),DIFFYS
      WRITE(LUOUT,*)

* Set X to X = R again by adding R0 
      DO 201 I=1,NDATA
        X(I)=X(I)+R0
201   CONTINUE

      C0=A(1)
      C1=A(2)
      C2=A(3)
      C3=A(4)
      C4=A(5)

      RETURN
      END
