      SUBROUTINE READIN(LUIN,LUOUT)
********************************************************************************
*                                                                              *
*  Copyright (C) Dage Sundholm, sundholm@chem.helsinki.fi                      *
*  Department of Chemistry, University of Helsinki                             *
*  P.O. Box 55, FIN-00014 Helsinki, Finland                                    *
*                                                                              *
*  This program is free software; you can redistribute it and/or               *
*  modify it under the terms of the GNU General Public License                 *
*  as published by the Free Software Foundation; either version 2              *
*  of the License, or (at your option) any later version.                      *
*                                                                              *
*  This program is distributed in the hope that it will be useful,             *
*  but WITHOUT ANY WARRANTY; without even the implied warranty of              *
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
*  GNU General Public License for more details.                                *
*                                                                              *
*  You should have received a copy of the GNU General Public License           *
*  along with this program; if not, write to the Free Software                 *
*  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. *
*                                                                              *
********************************************************************************
*
*
* Input for ro-vibrational average program for diatomic molecules 
* Written by Dage Sundholm 
* June 1994
*
* Very much inspired by the LUCAS input written by Jeppe Olsen 
*
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
*. General output
*. Local scratch
      PARAMETER(MXPLNC = 72 )
      CHARACTER*72 TITLEC
      CHARACTER*72 CARD,STRING
      COMMON/INPUT/ZETA1,ZETA2,AMASS1,AMASS2,REDMAS,REQUI,
     &             RINFIN,TEMPER(1000),NTEMP,IODD,
     &             ITRANS,MINVVA,MAXVVA,MINJVA,MAXJVA,
     &             NPOINT,NCELL,NDIM,IMESH,IVAR,NSAMPL
      CHARACTER*6 KEYWOR
      PARAMETER(MXPKW = 14)
      DIMENSION KEYWOR(MXPKW)
      DIMENSION ISETKW(MXPKW)
      DIMENSION TITLEC(3)
*
      DATA KEYWOR/'TITLE ','DIATOM','ISOTOP','VIBLEV','ROTLEV',
     &            'INFINI','NPOINT','NCELL ','MESH  ','CENTER',
     &            'TRANSI','RE    ','DPOINT','TEMP'/
*
*.Print flags
*
      NERROR =  0
      NWARN  =  0
      IECHO  =  0

*****************************************************************
*                                                               *
* Part 1 : Read in Keywords and perform some preliminary checks *
*                                                               *
*****************************************************************
*
      CALL ISETVC(ISETKW,0,MXPKW)
 1000 CONTINUE
*. Next potential keyword
        READ(LUIN,'(A)') CARD
*. Left-position nonblank characters in CARD
        CALL LFTPOS(CARD,MXPLNC)
*. Change to upper case
        CALL UPPCAS(CARD,MXPLNC)
        IF(CARD(1:1).EQ.'*') THEN
*. Skip comment cards
          GOTO 999
*. End of input card
        ELSE IF(CARD(1:5).EQ.'ENDOF') THEN
          GOTO 1001
        ELSE IF(CARD(1:1).NE.'.') THEN
*. Line out of context
          WRITE(LUOUT,'(1H ,A)') ' Warning, Data card out of context : '
          WRITE(LUOUT,'(1H ,A)') CARD
          NWARN = NWARN + 1
        ELSE IF(CARD(1:1).EQ.'.') THEN
*. A keyword has been identified, match with possible keywords
*
*
* =========================
*. Turn echo of keywords on
* =========================
*
          IF(CARD(2:5).EQ.'ECHO') THEN
            IECHO = 1
            WRITE(LUOUT,*) ' Echo of keywords turned on '
            GOTO 999
          END IF
*
* ==========================
*. Turn echo of keywords off
* ==========================
*
          IF(CARD(2:7).EQ.'NOECHO') THEN
            IECHO = 0
            WRITE(LUOUT,*) ' Echo of keywords turned off '
            GOTO 999
          END IF
*. Echo card
          IF(IECHO.EQ.1) WRITE(LUOUT,'(A)') CARD
*
          IF(CARD(2:6).EQ.'TITLE' ) THEN
*
* =========================
*.Keyword 1 :  TITLE cards
* =========================
*
*. Three title cards
            ISETKW(1) = 1
            DO 10 IC = 1, 3
              READ(LUIN,'(A)') TITLEC(IC)
   10       CONTINUE
            GOTO 999
          END IF
*
* ===============================
*.Keyword 2 :  Diatomic molecule   
* ===============================
*
* The nuclear charges            
          IF(CARD(2:7).EQ.'DIATOM') THEN
            ISETKW(2) = 1
              READ(LUIN,*) ZETA1,ZETA2  
              IERR=0
              IF(ZETA1.LT.0.D0) IERR=IERR+1
              IF(ZETA2.LT.0.D0) IERR=IERR+1          
              IF(IERR.GT.0) THEN
                WRITE(LUOUT,*) '  Input error !!!!! '
                WRITE(LUOUT,*) '  This molecule is at most an atom ' 
                WRITE(LUOUT,*) '  The given Z1 and Z2 are: ',
     &                            NINT(ZETA1),NINT(ZETA2) 
                NERROR = NERROR + 1
                ISETKW(2) = -1
              END IF
            GOTO 999
          END IF
*
* ===============================
*.Keyword 3 :  Isotops             
* ===============================
*
* The nuclear masses        
          IF(CARD(2:7).EQ.'ISOTOP') THEN
            ISETKW(3) = 1
              READ(LUIN,*) AMASS1,AMASS2  
              IERR=0
              IF(AMASS1.LT.0.D0) IERR=IERR+1
              IF(AMASS2.LT.0.D0) IERR=IERR+1       
              IF(IERR.GT.0) THEN
                WRITE(LUOUT,*) '  Input error !!!!! '
                WRITE(LUOUT,*) '  One of the atoms has negative'
                WRITE(LUOUT,*) '  or zero mass '
                WRITE(LUOUT,*) '  The given masses are : ',
     &                            AMASS1,AMASS2          
                NERROR = NERROR + 1
                ISETKW(3) = -1
              END IF
            GOTO 999
          END IF
*
* ===============================
*.Keyword 4 :  Vibrational symmetry
* ===============================
*
* Maximum and minimum v-values
          IF(CARD(2:7).EQ.'VIBLEV') THEN
            ISETKW(4) = 1
              READ(LUIN,*) MINVVA,MAXVVA
* A shift so that v=0 will become the first eigenvector  
              MINVVA=MINVVA+1
              MAXVVA=MAXVVA+1
              IERR=0
              IF(MINVVA.LT.1) IERR=IERR+1
              IF(MAXVVA.LT.1) IERR=IERR+1
              IF(MINVVA.GT.MAXVVA) IERR=IERR+1
              IF(IERR.GT.0) THEN
                WRITE(LUOUT,*) '  Input error !!!!! '
                WRITE(LUOUT,*) '  Input vibrational quantum numbers ',
     &                            MINVVA-1,MAXVVA-1
                WRITE(LUOUT,*) '  Are not valid           '
                NERROR = NERROR + 1
                ISETKW(4) = -1
              END IF
            GOTO 999
          END IF
*
* ===============================
*.Keyword 5 :  Rotational symmetry 
* ===============================
*
* Maximum and minimum J-values 
          IF(CARD(2:7).EQ.'ROTLEV') THEN
            ISETKW(5) = 1
              READ(LUIN,*) MINJVA,MAXJVA
              IERR=0
              IF(MINJVA.LT.0) IERR=IERR+1     
              IF(MAXJVA.LT.0) IERR=IERR+1          
              IF(MINJVA.GT.MAXJVA) IERR=IERR+1
              IF(IERR.GT.0) THEN
                WRITE(LUOUT,*) '  Input error !!!!! '
                WRITE(LUOUT,*) '  Input rotational quantum numbers ', 
     &                            MINJVA,MAXJVA 
                WRITE(LUOUT,*) '  Are not valid           '
                NERROR = NERROR + 1
                ISETKW(5) = -1
              END IF
            GOTO 999
          END IF
*
* ===================================================================
*.Keyword 6 :  Practical infinity ( does not need to be too accurate
* ===================================================================
*
          IF(CARD(2:7).EQ.'INFINI') THEN
            ISETKW(6) = 1
            READ(LUIN,*) RINFIN
            GOTO 999
          END IF
*
* ===========================================
*.Keyword 7 :  Number of element functions  
* ===========================================
*
          IF(CARD(2:7).EQ.'NPOINT') THEN
            ISETKW(7) = 1
            READ(LUIN,*) NPOINT 
            GOTO 999
          END IF
*
* ==================================================
*.Keyword 8 :  Total number of elements to be included
* ==================================================
*
          IF(CARD(2:6).EQ.'NCELL') THEN
            ISETKW(8) = 1
            READ(LUIN,*) NCELL
            GOTO 999
          END IF
*
* ===================================
*.Keyword 9 :  Way of choosing grid
* ===================================
* = 1 : equidistant grid 
* = 2 : exponential grid 
* = 3 : Biexponential grid distribution 
          IF(CARD(2:5).EQ.'MESH') THEN
              ISETKW(9) = 1
              READ(LUIN,*) IMESH
              GOTO 999
            END IF
          END IF
*
* =============================================
*.Keyword 10 :  Point of expansion in each cell
* =============================================
*
          IF(CARD(2:7).EQ.'CENTER') THEN
            ISETKW(10) = 1
            READ(LUIN,*) IVAR 
            GOTO 999
          END IF
*
* =================================================================
*.Keyword 11 :  Input channels for potential and property functions
* =================================================================
*
          IF(CARD(2:7).EQ.'TRANSI') THEN
            ISETKW(11) = 1
            ITRANS=1                 
            GOTO 999
          END IF
*
* =================================================================
*.Keyword 12 :  Estimated R_e for grid construction using IMESH=3
* =================================================================
*
          IF(CARD(2:3).EQ.'RE') THEN
            ISETKW(12) = 1
            READ(LUIN,*) REQUI 
            GOTO 999
          END IF
*
* ========================================
*.Keyword 13 :  The number of data points 
* ========================================
*
          IF(CARD(2:7).EQ.'DPOINT') THEN
            ISETKW(13) = 1
            READ(LUIN,*) NSAMPL
            GOTO 999
          END IF

*
* ========================================
*.Keyword 14 :  Temperature dependence   
* ========================================
*
          IF(CARD(2:5).EQ.'TEMP') THEN
            ISETKW(14) = 1
            READ(LUIN,*) NTEMP
            IF(NTEMP.GT.1000) THEN
              WRITE(LUOUT,*) ' Too many tempertures'
              WRITE(LUOUT,*) ' Stop in READIN '
              STOP
            END IF
            DO I=1,NTEMP
              READ(LUIN,*) TEMPER(I)
            END DO

* Temperature average over all J, all odd J, or all even J ?
            READ(LUIN,'(A)') STRING
            CALL LFTPOS(STRING,MXPLNC)
            CALL UPPCAS(STRING,MXPLNC)
            IODD=0
            IF(STRING(1:3).EQ.'EVE') IODD=2
            IF(STRING(1:3).EQ.'ODD') IODD=1
            GOTO 999
          END IF
*
*. KEYWORD was not identified
*
          WRITE(LUOUT,*) ' Unidentified KEYWORD card '
          WRITE(LUOUT,'(1H ,A)') CARD
          NERROR = NERROR + 1
*
  999   CONTINUE
      GOTO 1000
*.End of loop over KEYWORDS
 1001 CONTINUE
*
      IF(NERROR.NE.0) THEN
        WRITE(LUOUT,'(A)')
     &  ' Input will be aborted due to input errors '
        WRITE(LUOUT,'(A,I9)')
     &  ' Number of input errors detected in READIN ', NERROR
        WRITE(LUOUT,*) ' ERRORS occured for the following KEYWORDS'
        DO 50 IENTRY = 1, MXPKW
           IF(ISETKW(IENTRY).EQ.-1) WRITE(LUOUT,'(A)') KEYWOR(IENTRY)
   50   CONTINUE
        STOP 5
      END IF

*
* ==============================================================
*
* Part 2
*
* Check input, ensure that all neccessary keywords have been set,
* provide defaults for unnneccessary keywords
*
* ===============================================================
*
      NMISS = 0
*
*
*.1 : Default title
*
      IF(ISETKW(1).EQ.0) THEN
        TITLEC(1) =
     &  ' The rovibrational average will be calculated for  '
        TITLEC(2) =
     &  ' some property of a diatomic molecule by    '
        TITLEC(3) =
     &  ' someone who is too lazy to supply a TITLE                 '
        ISETKW(1) = 2
      END IF
*
*.2 : Missing nuclear charge
*
      IF(ISETKW(2).EQ.0) THEN
        NMISS = NMISS + 1
        WRITE(LUOUT,*)
     &  '  Input error ! .DIATOM ( nuclear charges ) must be specified '
        ZETA1 = 0.0D0
        ZETA2 = 0.0D0
      END IF
*
*.3 : Default nuclear masses
*
      IF(ISETKW(3).EQ.0) THEN
        ISETKW(3)=2 
        CALL ISOTOP(NINT(ZETA1),AMASS1,LUOUT)
        CALL ISOTOP(NINT(ZETA2),AMASS2,LUOUT) 
*       WRITE(LUOUT,*) ' The nuclear masses of the most abundant' 
*       WRITE(LUOUT,*) ' isotops are assumed' 
      END IF

*
*.4 : The default vibrational levels
*
      IF(ISETKW(4).EQ.0) THEN
        ISETKW(4) = 2
        MINVVA=1
        MAXVVA=10
      END IF

*
*.5 : The default rotational levels
*
      IF(ISETKW(5).EQ.0) THEN
        ISETKW(5) = 2
        MINJVA=0
        MAXJVA=4
      END IF

*
*.6 : The practical infinity 
*
      IF(ISETKW(6).EQ.0) THEN
        ISETKW(6) = 2
        RINFIN=30.D0 
      END IF

*
*.7  : Number of intervals per cell
*
      IF(ISETKW(7).EQ.0) THEN
        NPOINT = 7
        ISETKW(7) = 2
      END IF
*
*
*.8  : Total number of cells
*
      IF(ISETKW(8).EQ.0) THEN
        NCELL = 15
        ISETKW(8) = 2
      END IF
*
*.9  : Algorithm for selecting mesh
*
      IF(ISETKW(9).EQ.0) THEN
*. Default is a Biexponential Mesh 
        IMESH = 3
        ISETKW(9) = 2
      END IF
*
*.10 : Point of expansion for LIPS
*
      IF(ISETKW(10).EQ.0) THEN
*. Default is as close to midpoint as possible
        IVAR = (NPOINT-1)/2
        ISETKW(10) = 2
      END IF
*
*.11 : A Transition property                   
*
      IF(ISETKW(11).EQ.0) THEN
        ITRANS=0
        ISETKW(11) = 2
      END IF

*
*.12 : The mesh RE should be given somewhere 
*
      IF(ISETKW(12).EQ.0) THEN
        REQUI = 2.D0
        ISETKW(12) = 2
      END IF
*
*.13 : The number of sample points 
*
      IF(ISETKW(13).EQ.0) THEN
        ISETKW(13) = 2
        NMISS = NMISS + 1
        WRITE(LUOUT,*)
     &  '  Input error ! .DPOINT ',
     &  'The number of data points be specified '
        NSAMPL=0
      END IF

*
*.14 : The temperature average    
*
      IF(ISETKW(14).EQ.0) THEN
        ISETKW(14) = 2
        NTEMP=0
      END IF

      IF(NMISS.NE.0) THEN
        WRITE(LUOUT,'(1H ,A,I9)')
     &  ' Number of missing required keyword ', NMISS
        WRITE(LUOUT,*)
     &  ' You have wounded me I give up '
        STOP
      END IF

************************************************************
*                                                          *
* Part 3 : Print input                                     *
*                                                          *
************************************************************
*
      CALL HEADER
      WRITE(LUOUT,*)
      WRITE(LUOUT,*)
      WRITE(LUOUT,*) '******************'
      WRITE(LUOUT,*) '*  Title of run  *'
      WRITE(LUOUT,*) '******************'
      WRITE(LUOUT,*)
      CALL PRTITL(TITLEC)

      WRITE(LUOUT,*)
      WRITE(LUOUT,*) '************************************'
      WRITE(LUOUT,*) '* Specification of physical system *'
      WRITE(LUOUT,*) '************************************'
*
      WRITE(LUOUT,*)
      WRITE(LUOUT,'(8X,A,F8.3)') 
     & 'Nuclear charge of nucleus 1  ' ,ZETA1
      WRITE(LUOUT,*)
      WRITE(LUOUT,'(8X,A,F8.3)')
     & 'Nuclear charge of nucleus 2  ' ,ZETA2
*
      WRITE(LUOUT,*)
      WRITE(LUOUT,'(8X,A,F12.6)')
     & 'Nuclear mass of nucleus 1    ' ,AMASS1
      WRITE(LUOUT,*)
      WRITE(LUOUT,'(8X,A,F12.6)')
     & 'Nuclear mass of nucleus 2    ' ,AMASS2
      REDMAS=AMASS1*AMASS2/(AMASS1+AMASS2)
*  Reduced mass in au
*  (proton mass(in au)/electron mass(in au)/proton mass(in mass units)  
*   1836.1528/1.007825=1821.8965)
      REDMAS=REDMAS*1821.8965
      WRITE(LUOUT,*)
      WRITE(LUOUT,'(8X,A,D14.8)')
     & 'The reduced mass in au         ',REDMAS
*
      WRITE(LUOUT,*)
      WRITE(LUOUT,'(8X,A,I3)') 'Minimum vibrational level  ',MINVVA-1
      WRITE(LUOUT,'(8X,A,I3)') 'Maximum vibrational level  ',MAXVVA-1
      WRITE(LUOUT,'(8X,A,I3)') 'Minimum rotational level   ',MINJVA
      WRITE(LUOUT,'(8X,A,I3)') 'Maximum rotational level   ',MAXJVA
      WRITE(LUOUT,*) 
 
      IF(ITRANS.EQ.1) THEN
        WRITE(LUOUT,*)
        WRITE(LUOUT,'(8X,A)') 'A transition property calculation'
        WRITE(LUOUT,*)
      END IF

      IF(NTEMP.GT.0) THEN
        WRITE(LUOUT,*)
        WRITE(LUOUT,'(8X,A)') 'A temperature average calculation'
        IF(IODD.EQ.0) THEN
        WRITE(LUOUT,'(8X,A)') 'All rotational levels contribute'
        END IF
        IF(IODD.EQ.1) THEN
        WRITE(LUOUT,'(8X,A)') 'Only odd rotational levels contribute'
        END IF
        IF(IODD.EQ.2) THEN
        WRITE(LUOUT,'(8X,A)') 'Only even rotational levels contribute'
        END IF
        WRITE(LUOUT,*)
      END IF

*
      WRITE(LUOUT,*)
      WRITE(LUOUT,*) '***********************************'
      WRITE(LUOUT,*) '*  Finite element characteristics * '
      WRITE(LUOUT,*) '***********************************'
      WRITE(LUOUT,*)
*
      WRITE(LUOUT,1100) RINFIN
 1100 FORMAT('       The assumed practical infinity ',F10.4 )
      WRITE(LUOUT,1110 ) NPOINT 
 1110 FORMAT('       Number of element functions     ',I4)
      WRITE(LUOUT,1111 ) NCELL  
 1111 FORMAT('       Number of elements              ',I4)
*. Total number of gridpoints
      NDIM  = (NPOINT-1)*NCELL + 1
      WRITE(LUOUT,1112) NDIM 
 1112 FORMAT('       Number of grid points           ',I4)
      WRITE(LUOUT,1113) IVAR 
 1113 FORMAT('       Center of expansion in cell     ',I4)
      IF( IMESH .EQ. 1 ) WRITE(LUOUT,1140)
      IF( IMESH .EQ. 2 ) WRITE(LUOUT,1150)
      IF( IMESH .EQ. 3 ) WRITE(LUOUT,1160)
 1140 FORMAT('       Grid equidistant  in R ')
 1150 FORMAT('       Grid equidistant in log(R) ')
 1160 FORMAT('       Biexponential distribution of elements')  
      WRITE(LUOUT,*)
      WRITE(LUOUT,*)

      RETURN
      END 
