      SUBROUTINE SETMAT(A,S,V,W,A1,A2,A3,A4,NCELL,NPOINT,NDIM)
********************************************************************************
*                                                                              *
*  Copyright (C) Dage Sundholm, sundholm@chem.helsinki.fi                      *
*  Department of Chemistry, University of Helsinki                             *
*  P.O. Box 55, FIN-00014 Helsinki, Finland                                    *
*                                                                              *
*  This program is free software; you can redistribute it and/or               *
*  modify it under the terms of the GNU General Public License                 *
*  as published by the Free Software Foundation; either version 2              *
*  of the License, or (at your option) any later version.                      *
*                                                                              *
*  This program is distributed in the hope that it will be useful,             *
*  but WITHOUT ANY WARRANTY; without even the implied warranty of              *
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
*  GNU General Public License for more details.                                *
*                                                                              *
*  You should have received a copy of the GNU General Public License           *
*  along with this program; if not, write to the Free Software                 *
*  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. *
*                                                                              *
********************************************************************************
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      COMMON/IO/LUIN,LUOUT
 
*  PARAMETER LIST:
*
*  A      = Full Laplacian matrix (0.5*(-nabla**2))
*  S      = Full metric matrix     
*  W      = Full -1/r matrix
*  V      = Full centrifugal potential matrix (1/(r**2)) 
*  A1     = Matrix elements of type INTEGRAL( F(I)*F(J) )
*  A2     = Matrix elements of type INTEGRAL( F(I)*F(J)/r )
*  A3     = Matrix elements of type INTEGRAL( dF(I)*dF(J)/dr2 )
*  A4     = Matrix elements of type INTEGRAL( F(I)*F(J)/r2 )
*  NCELL  = Number of cells
*  NPOINT = Number of points per cell
*  NDIM   = Total number of points (NPOINT-1)*NCELL+1
 
      DIMENSION A(NDIM-2,NDIM-2)
      DIMENSION S(NDIM-2,NDIM-2)
      DIMENSION V(NDIM-2,NDIM-2) 
      DIMENSION W(NDIM-2,NDIM-2)
      DIMENSION A1(NPOINT*NPOINT*NCELL)          
      DIMENSION A2(NPOINT*NPOINT*NCELL)          
      DIMENSION A3(NPOINT*NPOINT*NCELL)          
      DIMENSION A4(NPOINT*NPOINT*NCELL)          
 
      NTEST=0

C Reset A-matrix, S-matrix, V-matrix and W-matrix
 
      CALL VCLR(A,1,(NDIM-2)*(NDIM-2))
      CALL VCLR(S,1,(NDIM-2)*(NDIM-2))
      CALL VCLR(V,1,(NDIM-2)*(NDIM-2))
      CALL VCLR(W,1,(NDIM-2)*(NDIM-2))
 
C 1) -1/2 nabla**2 matrix into A 
 
      FACTOR=0.5D0       
      CALL MADBL1(A,A3,NDIM,NPOINT,NCELL,FACTOR)
      IF(NTEST.GT.0) THEN 
        WRITE(LUOUT,*) 
        WRITE(LUOUT,'(A)') ' The A-matrix' 
        WRITE(LUOUT,*) NDIM 
        WRITE(LUOUT,*) 
        CALL WRTMAT(A,NDIM-2,NDIM-2,NDIM-2,NDIM-2)
      END IF
 
C 2)  Centrifugal potential matrix into (1/(r**2)) V 
 
      FACTOR=0.5D0
      CALL MADBL1(V,A4,NDIM,NPOINT,NCELL,FACTOR)

      IF(NTEST.GT.0) THEN
        WRITE(LUOUT,*)
        WRITE(LUOUT,'(A)') ' The V-matrix'
        WRITE(LUOUT,*) NDIM
        WRITE(LUOUT,*)
        CALL WRTMAT(V,NDIM-2,NDIM-2,NDIM-2,NDIM-2)
      END IF

C 3)  -1/r potential matrix into W

      FACTOR=-1.0 
      CALL MADBL1(W,A2,NDIM,NPOINT,NCELL,FACTOR)
    
      IF(NTEST.GT.0) THEN 
        WRITE(LUOUT,*) 
        WRITE(LUOUT,'(A)') ' The W-matrix' 
        WRITE(LUOUT,*) NDIM 
        WRITE(LUOUT,*) 
        CALL WRTMAT(W,NDIM-2,NDIM-2,NDIM-2,NDIM-2)
      END IF

C 4) The S-Matrix
 
      FACTOR=1.D0
      CALL MADBL1(S,A1,NDIM,NPOINT,NCELL,FACTOR)
      IF(NTEST.GT.0) THEN
        WRITE(LUOUT,*) 
        WRITE(LUOUT,'(A)') ' The S-matrix' 
        WRITE(LUOUT,*) NDIM 
        WRITE(LUOUT,*)
        CALL WRTMAT(S,NDIM-2,NDIM-2,NDIM-2,NDIM-2)
      END IF
 
      RETURN
      END
