      SUBROUTINE SPLINE(X,Y,NMAX,YP1,YPN,HELP,Y2)
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION X(NMAX),Y(NMAX),HELP(NMAX),Y2(NMAX)
*
*  INPUT:  X(I)    : The X-values
*          Y(I)    : The Y-values
*          NMAX    : Number of X,Y pairs
*          YP1     : First derivative of the function at point 1
*          YPN     : First derivative of the function at point NMAX
*          HELP(I) : A work vector of length NMAX
*
*  OUTPUT: Y2(I)   : The second derivative of the function for 
*                    each X-value
*
*
*  Numerical Recipes page 88
*  Copied by Dage Sundholm, 16.6.1995, debugged 22.6.1995 ok
*
* Boundary condition:
* IF first derivative is too large I set the second derivative to zero
*
      N=NMAX
      DERMAX=1.D30
      IF(ABS(YP1).GT.DERMAX) THEN
        Y2(1)   = 0.D0
        HELP(1) = 0.D0
      ELSE
        Y2(1)=-0.5D0
        HELP(1) = (3.D0/(X(2)-X(1)))*((Y(2)-Y(1))/(X(2)-X(1))-YP1)
      END IF

      DO 100 I=2,N-1
        SIG     = (X(I)-X(I-1))/(X(I+1)-X(I-1))
        P       = SIG*Y2(I-1)+2.D0
        Y2(I)   = (SIG-1.D0)/P
        HELP(I) = (6.D0*((Y(I+1)-Y(I))/(X(I+1)-X(I))-(Y(I)-Y(I-1))/
     &            (X(I)-X(I-1)))/(X(I+1)-X(I-1))-SIG*HELP(I-1))/P
100   CONTINUE

      IF(ABS(YPN).GT.DERMAX) THEN
        QN = 0.D0
        UN = 0.D0
      ELSE
        QN = 0.5D0
        UN= (3.D0/(X(N)-X(N-1)))*(YPN-(Y(N)-Y(N-1))/(X(N)-X(N-1)))
      END IF

      Y2(N) = (UN-QN*HELP(N-1))/(QN*Y2(N-1)+1.D0)
      DO 200 I=N-1,1,-1
        Y2(I) = Y2(I)*Y2(I+1)+HELP(I)
200   CONTINUE

      RETURN
      END
