      SUBROUTINE TOTHAM(A,S,W,V,C,T,SQU,RS,H,
     &                  QUANTU,REDMAS,NCELL,NPOINT,NDIM)
********************************************************************************
*                                                                              *
*  Copyright (C) Dage Sundholm, sundholm@chem.helsinki.fi                      *
*  Department of Chemistry, University of Helsinki                             *
*  P.O. Box 55, FIN-00014 Helsinki, Finland                                    *
*                                                                              *
*  This program is free software; you can redistribute it and/or               *
*  modify it under the terms of the GNU General Public License                 *
*  as published by the Free Software Foundation; either version 2              *
*  of the License, or (at your option) any later version.                      *
*                                                                              *
*  This program is distributed in the hope that it will be useful,             *
*  but WITHOUT ANY WARRANTY; without even the implied warranty of              *
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
*  GNU General Public License for more details.                                *
*                                                                              *
*  You should have received a copy of the GNU General Public License           *
*  along with this program; if not, write to the Free Software                 *
*  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. *
*                                                                              *
********************************************************************************
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      COMMON/IO/LUIN,LUOUT
      DIMENSION A(NDIM-2,NDIM-2),S(NDIM-2,NDIM-2),V(NDIM-2,NDIM-2)
      DIMENSION C(NDIM-2,NDIM-2),T(NDIM-2,NDIM-2),SQU(NDIM-2) 
      DIMENSION RS(NCELL),H(NCELL) 

* Debugging switch
      IDEBUG=0

* Set a ZETA for tesing purposes, normally ZETA=0.D0
      IF(IDEBUG.GT.0) THEN 
        ZETA=1.D0
      ELSE 
        ZETA=0.D0
      END IF

      IF(ZETA.NE.0.D0) THEN
        REDMAS=1.D0
        WRITE(LUOUT,*)
        WRITE(LUOUT,'(A)') ' The debugging switch is on in TOTHAM'
        WRITE(LUOUT,'(A)') ' The reduced mass is set to 1 in TOTHAM'
        WRITE(LUOUT,*)
      END IF
      
* 1)  Multiply A by 1/REDMAS
* T = -0.5* (nabla**2)/REDMAS 
      FACTOR=1.D0/REDMAS
      CALL VCLR(T,1,(NDIM-2)*(NDIM-2))
      CALL DAXPY((NDIM-2)*(NDIM-2),FACTOR,A,1,T,1)

* 2) Add the centrifugal potential 
* T = T + J*(J+1)/2/REDMAS  
      FACTOR=QUANTU*(QUANTU+1)/REDMAS
      IF(FACTOR.NE.0.D0) THEN 
       CALL DAXPY((NDIM-2)*(NDIM-2),FACTOR,V,1,T,1)
      END IF
   
* 3) If we want the H-like atom then 
*  T = T + ZETA * (-1/r)
      IF(ZETA.NE.0.D0) THEN
        WRITE(LUOUT,*)
        WRITE(LUOUT,'(A)') ' The Hamilton operator for H-atom is set'
        WRITE(LUOUT,*)
        FACTOR=ZETA
        CALL DAXPY((NDIM-2)*(NDIM-2),FACTOR,W,1,T,1)
      END IF
      
* 4) An external and arbitrary potential.
*    However (1/r)- or (r)-like potentials should be avoided
*    because they cause numerical difficulties
*  T = T + C    or            
*  T = T + (U**0.5) * S * (U**0.5)
*  Of course must U alway be positive (That is set in POTREA)
      IF(ZETA.EQ.0.D0) THEN
        CALL VCLR(C,1,(NDIM-2)*(NDIM-2))
        CALL SUMAT(SQU,S,C,NCELL,NPOINT,NDIM) 
        FACTOR=1.D0
        CALL DAXPY((NDIM-2)*(NDIM-2),FACTOR,C,1,T,1)
      END IF

      RETURN
      END
