***********************************************************************
*                                                                     *
*   THIS CODE HAS BEEN ALMOST DIRECTLY COPIED FROM                    *
*   M. MASAMURA, J.COMP.CHEM 9 (1988) 257.                            *
*   BY DAGE SUNDHOLM  (29.4.1988)                                     *
*                                                                     *
***********************************************************************

      SUBROUTINE TRANSH(N,H,S,P,WORK)
********************************************************************************
*                                                                              *
*  Copyright (C) Dage Sundholm, sundholm@chem.helsinki.fi                      *
*  Department of Chemistry, University of Helsinki                             *
*  P.O. Box 55, FIN-00014 Helsinki, Finland                                    *
*                                                                              *
*  This program is free software; you can redistribute it and/or               *
*  modify it under the terms of the GNU General Public License                 *
*  as published by the Free Software Foundation; either version 2              *
*  of the License, or (at your option) any later version.                      *
*                                                                              *
*  This program is distributed in the hope that it will be useful,             *
*  but WITHOUT ANY WARRANTY; without even the implied warranty of              *
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
*  GNU General Public License for more details.                                *
*                                                                              *
*  You should have received a copy of the GNU General Public License           *
*  along with this program; if not, write to the Free Software                 *
*  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. *
*                                                                              *
********************************************************************************
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)

C Symmetric matrices are assumed
C Transform H to H' obtain the transformation matrix P
C (HC=ESC) => (H'C'=EC') and (C=PC')

C N     : Dimension of the problem
C H     : Hamilton matrix, in H out H' (full matrix)
C S     : Overlap matrix,  in S out I  (full matrix)
C P     : Transformation matrix in trash out P (full matrix)

      DIMENSION S(N,N),H(N,N),P(N,N),WORK(N)

C Neglect matrix elements less than DEPS

      DEPS=0.5D-14
      ONE=1.

C Set P to unit matrix

      CALL VCLR(P,1,N*N)
      CALL SETDIA(ONE,P,N)

C First part of the transformation of the H and P matrices

      DO 20 K=1,N-1
        DO 20 J=N,K+1,-1

          D=S(K,J)/S(K,K)
          IF(ABS(D).GT.DEPS) THEN

            DO 30 I=K+1,J
30          S(I,J)=S(I,J)-D*S(K,I)
            DO 31 I=K+1,J
31          H(I,J)=H(I,J)-D*H(K,I)

            DO 40 I=1,K
40          H(I,J)=H(I,J)-D*H(I,K)

            DO 50 I=J,N
50          H(J,I)=H(J,I)-D*H(K,I)

            DO 60 I=1,K
60          P(I,J)=P(I,J)-D*P(I,K)

          END IF
20    CONTINUE

C Second part of the transformation obtaining the final H and P matrices
C but just the upper triangle.

      DO 70 I=1,N
        E=SQRT(S(I,I))

        DO 80 J=1,N
80      H(I,J)=H(I,J)/E

        DO 90 J=1,I
90      H(J,I)=H(J,I)/E

        DO 100 J=1,I
        P(J,I)=P(J,I)/E
100     CONTINUE
70    CONTINUE

C To be sure, copy the upper triangle to the lower triangle
C set the S matrix to be unit matrix
C (Just in case)

      DO 200 I=1,N-1
        DO 200 J=I+1,N
200     H(J,I)=H(I,J)

      CALL VCLR(S,1,N*N)
      CALL SETDIA(ONE,S,N)

      RETURN
      END
