      PROGRAM VIBROT
********************************************************************************
*                                                                              *
*  Copyright (C) Dage Sundholm, sundholm@chem.helsinki.fi                      *
*  Department of Chemistry, University of Helsinki                             *
*  P.O. Box 55, FIN-00014 Helsinki, Finland                                    *
*                                                                              *
*  This program is free software; you can redistribute it and/or               *
*  modify it under the terms of the GNU General Public License                 *
*  as published by the Free Software Foundation; either version 2              *
*  of the License, or (at your option) any later version.                      *
*                                                                              *
*  This program is distributed in the hope that it will be useful,             *
*  but WITHOUT ANY WARRANTY; without even the implied warranty of              *
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
*  GNU General Public License for more details.                                *
*                                                                              *
*  You should have received a copy of the GNU General Public License           *
*  along with this program; if not, write to the Free Software                 *
*  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. *
*                                                                              *
********************************************************************************
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)

*   Principal Author Dage Sundholm
*   Version 29 June 1995 

*   Main program for solving the ro-vibational averaging problem
*   for diatomic molecules.  The rigid-rotor and Born-Oppenheimer
*   approximations are assumed. 
*
*   The radial range is divided into a given number of elements and
*   each element consists of a given number of equidistant grid points.
*
*   The final equation is very reminiscent of 
*   The atomic radial one-electron Schroedinger equation 
 
*   Main program for constructing H and S matrices, and diagonalization
*   of the generalized eigenvalue problem Hx=ESx.
 
      COMMON/BLANK/ WORK (0:1024*1024-1)
      COMMON/IO/LUIN,LUOUT
      COMMON/INPUT/ZETA1,ZETA2,AMASS1,AMASS2,REDMAS,REQUI,
     &             RINFIN,TEMPER(1000),NTEMP,IODD,
     &             ITRANS,MINVVA,MAXVVA,MINJVA,MAXJVA,
     &             NPOINT,NCELL,NDIM,IMESH,IVAR,NSAMPL

* A debugging switch
      IDEBUG=0

* Input and output channels
      LUIN  = 5
      LUOUT = 6
 
*   Read input
*   ==========

      CALL READIN(LUIN,LUOUT)

*   Allocate memory space 
*   =====================

      CALL GETMEM('ALLO','REAL',IRS,NCELL)
      CALL GETMEM('ALLO','REAL',IH0,NCELL)
      CALL GETMEM('ALLO','REAL',KPOTE1,NDIM)  
      CALL GETMEM('ALLO','REAL',KPOTE2,NDIM)
      CALL GETMEM('ALLO','REAL',KPROP,NDIM)
      CALL GETMEM('ALLO','REAL',KEIGEN,NDIM*NDIM)
      CALL GETMEM('ALLO','REAL',KSMAT,NDIM*NDIM)
      CALL GETMEM('ALLO','REAL',ID1,100)
      CALL GETMEM('ALLO','REAL',ID2,100)
      CALL GETMEM('ALLO','REAL',ID3,100)
      CALL GETMEM('ALLO','REAL',ID4,100)
      CALL GETMEM('ALLO','REAL',ID5,NPOINT*NPOINT)
      CALL GETMEM('ALLO','REAL',ID6,NPOINT*NPOINT*(NPOINT-1))
      CALL GETMEM('ALLO','REAL',IA1,NPOINT*NPOINT*NCELL)
      CALL GETMEM('ALLO','REAL',IA2,NPOINT*NPOINT*NCELL)
      CALL GETMEM('ALLO','REAL',IA3,NPOINT*NPOINT*NCELL)
      CALL GETMEM('ALLO','REAL',IA4,NPOINT*NPOINT*NCELL)
      CALL GETMEM('ALLO','REAL',KAMAT,NDIM*NDIM)
      CALL GETMEM('ALLO','REAL',KVMAT,NDIM*NDIM)
      CALL GETMEM('ALLO','REAL',KCMAT,NDIM*NDIM)
      CALL GETMEM('ALLO','REAL',KTMAT,NDIM*NDIM)
      CALL GETMEM('ALLO','REAL',KEMAT,NDIM*NDIM)
      CALL GETMEM('ALLO','REAL',KWMAT,NDIM*NDIM)
      CALL GETMEM('ALLO','REAL',KFMAT,NDIM*NDIM)
      CALL GETMEM('ALLO','REAL',KWRK,NDIM)
      CALL GETMEM('ALLO','REAL',KSRAD,NSAMPL)
      CALL GETMEM('ALLO','REAL',KSPRO,NSAMPL)
      CALL GETMEM('ALLO','REAL',KSPOT1,NSAMPL)
      CALL GETMEM('ALLO','REAL',KSPOT2,NSAMPL)
      CALL GETMEM('ALLO','REAL',KSHEL1,NSAMPL)
      CALL GETMEM('ALLO','REAL',KSHEL2,NSAMPL)
      CALL GETMEM('ALLO','REAL',KCOORD,NDIM)
      CALL GETMEM('ALLO','REAL',KELEV,
     &           (MAXVVA-MINVVA+1)*(MAXJVA-MINJVA+1))
      CALL GETMEM('ALLO','REAL',KPROPE,
     &           (MAXVVA-MINVVA+1)*(MAXJVA-MINJVA+1))

* If necessary to check the memory allocation
      IF(IDEBUG.GT.10) THEN 
        CALL GETMEM('LIST','REAL',IDU,IDU)
      END IF 

*   Set coordinates
*   ===============

* linear grid
      IF(IMESH.EQ.1) THEN 
       CALL COORD1(WORK(IRS),WORK(IH0),NPOINT,NCELL,RINFIN,WORK(KCOORD))
      END IF
* exponential grid
      IF(IMESH.EQ.2) THEN 
       CALL COORD2(WORK(IRS),WORK(IH0),NPOINT,NCELL,RINFIN,WORK(KCOORD))
      END IF
* biexponential grid
      IF(IMESH.EQ.3) THEN
       CALL COORD3(WORK(IRS),WORK(IH0),NPOINT,NCELL,REQUI,
     &             RINFIN,WORK(KCOORD))
      END IF

*   Read the sample points and set potential and property functions  
*   ===============================================================

      CALL POTREA(WORK(KSRAD),WORK(KSPRO),WORK(KSPOT1),WORK(KSPOT2),
     &                        WORK(KPROP),WORK(KPOTE1),WORK(KPOTE2),
     &            WORK(KSHEL1),WORK(KSHEL2),
     &            WORK(KCOORD),NDIM,LUIN,LUOUT,NSAMPL,ITRANS,
     &            PSHIF1,PSHIF2) 

*   Solve the Schroedinger equation and integrate 
*   =============================================

*   For each J compute and diagonalize the Hamilton matrix
*   Calculated the rovibrational averages os only one electronic 
*   state.
      IF(ITRANS.EQ.0) THEN 
       DO 1000 J=MINJVA,MAXJVA
         QUANTU=1.D0*J
         CALL HASMAT(NPOINT,NCELL,NDIM,IVAR,WORK(IRS),WORK(IH0),
     &               WORK(KPOTE1),
     &               WORK(ID1),WORK(ID2),WORK(ID3),WORK(ID4),
     &               WORK(ID5),WORK(ID6),WORK(IA1),WORK(IA2),
     &               WORK(IA3),WORK(IA4),WORK(KAMAT),WORK(KVMAT),
     &               WORK(KCMAT),WORK(KTMAT),WORK(KEMAT),WORK(KWRK), 
     &               QUANTU,REDMAS,WORK(KPROP),MINVVA,MAXVVA,
     &               WORK(KWMAT),WORK(KFMAT),WORK(KEIGEN),WORK(KSMAT),
     &               PSHIF1,WORK(KELEV),WORK(KPROPE),MINJVA,MAXJVA) 
1000   CONTINUE 
      
        IF(NTEMP.GT.0) THEN
          CALL TEMPAV(WORK(KELEV),WORK(KPROPE),MINVVA,MAXVVA,
     &                MINJVA,MAXJVA,TEMPER,NTEMP,IODD)
        END IF 

      END IF 

*  For each J compute and diagonalize the Hamilton matrix
*  Store the vibrational functions for determination of 
*  transition properties between different electronic states.
      IF(ITRANS.EQ.1) THEN 

*  For calculation of transition properties we need some additional
*  work arrays, the allocation is below
        LENEVA=(NDIM-2)*(MAXVVA-MINVVA+1)*(MAXJVA-MINJVA+1)
        LENEVB=(NDIM-2)*(MAXVVA-MINVVA+1)*(MAXJVA-MINJVA+1)
        CALL GETMEM('ALLO','REAL',KEIGSA,LENEVA)
        CALL GETMEM('ALLO','REAL',KEIGSB,LENEVB)
        CALL GETMEM('ALLO','REAL',KSCR,NDIM*NDIM)
        CALL GETMEM('ALLO','REAL',KPROV,NDIM*NDIM)
        NUMINT=((MAXVVA-MINVVA+1)*(MAXJVA-MINJVA+1))**2
        CALL GETMEM('ALLO','REAL',KINTER,NUMINT)

* Solve the rovibrational problem for the first potential 
* save the rovibrational wave function for the calculation of 
* transition properties. (The left state)
        DO 2000 J=MINJVA,MAXJVA
         QUANTU=1.D0*J
         CALL HASMAT(NPOINT,NCELL,NDIM,IVAR,WORK(IRS),WORK(IH0),
     &               WORK(KPOTE1),
     &               WORK(ID1),WORK(ID2),WORK(ID3),WORK(ID4),
     &               WORK(ID5),WORK(ID6),WORK(IA1),WORK(IA2),
     &               WORK(IA3),WORK(IA4),WORK(KAMAT),WORK(KVMAT),
     &               WORK(KCMAT),WORK(KTMAT),WORK(KEMAT),WORK(KWRK),
     &               QUANTU,REDMAS,WORK(KPROP),MINVVA,MAXVVA,
     &               WORK(KWMAT),WORK(KFMAT),WORK(KEIGEN),WORK(KSMAT),
     &               PSHIF1,WORK(KELEV),WORK(KPROPE),MINJVA,MAXJVA)
         IPO=J*(NDIM-2)*(MAXVVA-MINVVA+1)
         IPOA=(NDIM-2)*(MINVVA-1)
         CALL VMOV(WORK(KEMAT+IPOA),1,WORK(KEIGSA+IPO),1,
     &             (NDIM-2)*(MAXVVA-MINVVA+1))
2000    CONTINUE
         
* Solve the rovibrational problem for the second potential 
* save the rovibrational wave function times the property times
* the overlap matrix for the calculation of transition properties. 
* (the right state) 
        DO 3000 J=MINJVA,MAXJVA
         QUANTU=1.D0*J
         CALL HASMAT(NPOINT,NCELL,NDIM,IVAR,WORK(IRS),WORK(IH0),
     &               WORK(KPOTE2),
     &               WORK(ID1),WORK(ID2),WORK(ID3),WORK(ID4),
     &               WORK(ID5),WORK(ID6),WORK(IA1),WORK(IA2),
     &               WORK(IA3),WORK(IA4),WORK(KAMAT),WORK(KVMAT),
     &               WORK(KCMAT),WORK(KTMAT),WORK(KEMAT),WORK(KWRK),
     &               QUANTU,REDMAS,WORK(KPROP),MINVVA,MAXVVA,
     &               WORK(KWMAT),WORK(KFMAT),WORK(KEIGEN),WORK(KSMAT),
     &               PSHIF2,WORK(KELEV),WORK(KPROPE),MINJVA,MAXJVA)
         CALL PROMUL(WORK(KEMAT),WORK(KPROP),NDIM,WORK(KSMAT),
     &               WORK(KSCR),WORK(KPROV))
         IPO=J*(NDIM-2)*(MAXVVA-MINVVA+1)
         IPOA=(NDIM-2)*(MINVVA-1)
         CALL VMOV(WORK(KPROV+IPOA),1,WORK(KEIGSB+IPO),1,
     &             (NDIM-2)*(MAXVVA-MINVVA+1))
3000    CONTINUE
                         
* Then it is time for the calculation of the average properties between
* the rovibrational wave functions of different electronic states.
* I do not know the symmetry of your transition operator,
* Your job is to select the desired terms (eg. J' = J+1 or J-1 for 
* dipole transitions.) I give you all the information

*  The rotational functions of the A state (the left state)
        DO 4000 JA=MINJVA,MAXJVA
*  The vibrational functions of the A state (the left state)
         DO 4000 IVIBA=MINVVA,MAXVVA
*  The rotational functions of the B state (the right state)
          DO 4000 JB=MINJVA,MAXJVA
*  The vibrational functions of the B state (the right state)
           DO 4000 IVIBB=MINVVA,MAXVVA 
             IA=(JA*(MAXVVA-MINVVA+1)+(IVIBA-MINVVA))*(NDIM-2)
             IB=(JB*(MAXVVA-MINVVA+1)+(IVIBB-MINVVA))*(NDIM-2)
             AVE=DDOT(NDIM-2,WORK(KEIGSA+IA),1,WORK(KEIGSB+IB),1)
             CALL AINSET(WORK(KINTER),AVE,MINVVA,MAXVVA,MINJVA,MAXJVA,
     &                   IVIBA,JA,IVIBB,JB)
4000    CONTINUE

*  Print out the transition property
      CALL PRIINT(WORK(KINTER),MINVVA,MAXVVA,MINJVA,MAXJVA,LUOUT)

      END IF

*   The end ( any memory problems ?) 
*   ================================

      CALL GETMEM('LIST','REAL',IDUMM,IDUMM)
 
      END
